﻿using System;
using System.Collections.Generic;

// Base class for model versions
abstract class ModelVersion
{
    public abstract void Load();
    public abstract void Predict();
}

// Concrete implementation for Model Version 1
class ModelVersion1 : ModelVersion
{
    public override void Load()
    {
        Console.WriteLine("Loading Model Version 1");
        // Load model version 1
    }

    public override void Predict()
    {
        Console.WriteLine("Predicting using Model Version 1");
        // Use model version 1 for prediction
    }
}

// Concrete implementation for Model Version 2
class ModelVersion2 : ModelVersion
{
    public override void Load()
    {
        Console.WriteLine("Loading Model Version 2");
        // Load model version 2
    }

    public override void Predict()
    {
        Console.WriteLine("Predicting using Model Version 2");
        // Use model version 2 for prediction
    }
}

// Model version manager
class ModelVersionManager
{
    private Dictionary<int, ModelVersion> modelVersions;

    public ModelVersionManager()
    {
        modelVersions = new Dictionary<int, ModelVersion>();
    }

    public void RegisterModelVersion(int version, ModelVersion model)
    {
        modelVersions[version] = model;
    }

    public void LoadModelVersion(int version)
    {
        if (modelVersions.TryGetValue(version, out ModelVersion model))
        {
            model.Load();
        }
        else
        {
            Console.WriteLine("Model version " + version + " not found.");
        }
    }

    public void PredictUsingModel(int version)
    {
        if (modelVersions.TryGetValue(version, out ModelVersion model))
        {
            model.Predict();
        }
        else
        {
            Console.WriteLine("Model version " + version + " not found.");
        }
    }
}

class ModelVersioning
{
    static void Main(string[] args)
    {
        // Create model versions
        ModelVersion1 modelVersion1 = new ModelVersion1();
        ModelVersion2 modelVersion2 = new ModelVersion2();

        // Create model version manager
        ModelVersionManager modelVersionManager = new ModelVersionManager();

        // Register model versions with the manager
        modelVersionManager.RegisterModelVersion(1, modelVersion1);
        modelVersionManager.RegisterModelVersion(2, modelVersion2);

        // Load and predict using specific model versions
        modelVersionManager.LoadModelVersion(1);
        modelVersionManager.PredictUsingModel(1);

        modelVersionManager.LoadModelVersion(2);
        modelVersionManager.PredictUsingModel(2);
    }
}
